@@ -22,7 +22,9 @@
 #include "config.h"
 
 #include <stdlib.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <string.h>
 
 #include "gtkapplication.h"
@@ -46,13 +48,12 @@
  *
  * Currently, GtkApplication handles GTK+ initialization, application
  * uniqueness, provides some basic scriptability by exporting 'actions',
- * implements some standard actions itself (such as 'Quit') and manages
- * a list of toplevel windows whose life-cycle is automatically tied to
- * the life-cycle of your application.
+ * and manages a list of toplevel windows whose life-cycle is automatically
+ * tied to the life-cycle of your application.
  *
  * <example id="gtkapplication"><title>A simple application</title>
  * <programlisting>
- * <xi:include xmlns:xi="http://www.w3.org/2001/XInclude" parse="text" href="../../../../gtk/tests/gtk-example-application.c">
+ * <xi:include xmlns:xi="http://www.w3.org/2001/XInclude" parse="text" href="../../../../examples/bloatpad.c">
  *  <xi:fallback>FIXME: MISSING XINCLUDE CONTENT</xi:fallback>
  * </xi:include>
  * </programlisting>
