@@ -48,16 +48,18 @@
  */
 
 #include "config.h"
-#include "gtkmain.h"
+
+#include <stdlib.h>
+#include <string.h>
+
+#include "gtkmainprivate.h"
 #include "gtktexttag.h"
 #include "gtktexttypes.h"
 #include "gtktexttagtable.h"
 #include "gtkintl.h"
 #include "gtkmarshalers.h"
 #include "gtkprivate.h"
-
-#include <stdlib.h>
-#include <string.h>
+#include "gtktypebuiltins.h"
 
 enum {
   EVENT,
@@ -665,12 +667,21 @@ gtk_text_tag_class_init (GtkTextTagClass *klass)
                   G_TYPE_OBJECT,
                   GDK_TYPE_EVENT | G_SIGNAL_TYPE_STATIC_SCOPE,
                   GTK_TYPE_TEXT_ITER);
+
+  g_type_class_add_private (klass, sizeof (GtkTextTagPrivate));
 }
 
 static void
 gtk_text_tag_init (GtkTextTag *text_tag)
 {
-  text_tag->values = gtk_text_attributes_new ();
+  GtkTextTagPrivate *priv;
+
+  text_tag->priv = G_TYPE_INSTANCE_GET_PRIVATE (text_tag,
+                                                GTK_TYPE_TEXT_TAG,
+                                                GtkTextTagPrivate);
+  priv = text_tag->priv;
+
+  priv->values = gtk_text_attributes_new ();
 }
 
 /**
@@ -695,20 +706,19 @@ gtk_text_tag_new (const gchar *name)
 static void
 gtk_text_tag_finalize (GObject *object)
 {
-  GtkTextTag *text_tag;
+  GtkTextTag *text_tag = GTK_TEXT_TAG (object);
+  GtkTextTagPrivate *priv = text_tag->priv;
 
-  text_tag = GTK_TEXT_TAG (object);
+  if (priv->table)
+    gtk_text_tag_table_remove (priv->table, text_tag);
 
-  if (text_tag->table)
-    gtk_text_tag_table_remove (text_tag->table, text_tag);
+  g_assert (priv->table == NULL);
 
-  g_assert (text_tag->table == NULL);
+  gtk_text_attributes_unref (priv->values);
+  priv->values = NULL;
 
-  gtk_text_attributes_unref (text_tag->values);
-  text_tag->values = NULL;
-  
-  g_free (text_tag->name);
-  text_tag->name = NULL;
+  g_free (priv->name);
+  priv->name = NULL;
 
   G_OBJECT_CLASS (gtk_text_tag_parent_class)->finalize (object);
 }
@@ -716,21 +726,23 @@ gtk_text_tag_finalize (GObject *object)
 static void
 set_bg_color (GtkTextTag *tag, GdkColor *color)
 {
+  GtkTextTagPrivate *priv = tag->priv;
+
   if (color)
     {
-      if (!tag->bg_color_set)
+      if (!priv->bg_color_set)
         {
-          tag->bg_color_set = TRUE;
+          priv->bg_color_set = TRUE;
           g_object_notify (G_OBJECT (tag), "background-set");
         }
-      
-      tag->values->appearance.bg_color = *color;
+
+      priv->values->appearance.bg_color = *color;
     }
   else
     {
-      if (tag->bg_color_set)
+      if (priv->bg_color_set)
         {
-          tag->bg_color_set = FALSE;
+          priv->bg_color_set = FALSE;
           g_object_notify (G_OBJECT (tag), "background-set");
         }
     }
@@ -739,20 +751,22 @@ set_bg_color (GtkTextTag *tag, GdkColor *color)
 static void
 set_fg_color (GtkTextTag *tag, GdkColor *color)
 {
+  GtkTextTagPrivate *priv = tag->priv;
+
   if (color)
     {
-      if (!tag->fg_color_set)
+      if (!priv->fg_color_set)
         {
-          tag->fg_color_set = TRUE;
+          priv->fg_color_set = TRUE;
           g_object_notify (G_OBJECT (tag), "foreground-set");
         }
-      tag->values->appearance.fg_color = *color;
+      priv->values->appearance.fg_color = *color;
     }
   else
     {
-      if (tag->fg_color_set)
+      if (priv->fg_color_set)
         {
-          tag->fg_color_set = FALSE;
+          priv->fg_color_set = FALSE;
           g_object_notify (G_OBJECT (tag), "foreground-set");
         }
     }
@@ -761,28 +775,30 @@ set_fg_color (GtkTextTag *tag, GdkColor *color)
 static void
 set_pg_bg_color (GtkTextTag *tag, GdkColor *color)
 {
+  GtkTextTagPrivate *priv = tag->priv;
+
   if (color)
     {
-      if (!tag->pg_bg_color_set)
+      if (!priv->pg_bg_color_set)
         {
-          tag->pg_bg_color_set = TRUE;
+          priv->pg_bg_color_set = TRUE;
           g_object_notify (G_OBJECT (tag), "paragraph-background-set");
         }
       else
-	gdk_color_free (tag->values->pg_bg_color);
+	gdk_color_free (priv->values->pg_bg_color);
 
-      tag->values->pg_bg_color = gdk_color_copy (color);
+      priv->values->pg_bg_color = gdk_color_copy (color);
     }
   else
     {
-      if (tag->pg_bg_color_set)
+      if (priv->pg_bg_color_set)
         {
-          tag->pg_bg_color_set = FALSE;
+          priv->pg_bg_color_set = FALSE;
           g_object_notify (G_OBJECT (tag), "paragraph-background-set");
-	  gdk_color_free (tag->values->pg_bg_color);
+	  gdk_color_free (priv->values->pg_bg_color);
         }
 
-      tag->values->pg_bg_color = NULL;
+      priv->values->pg_bg_color = NULL;
     }
 }
 
@@ -888,6 +904,7 @@ static void
 set_font_description (GtkTextTag           *text_tag,
                       PangoFontDescription *font_desc)
 {
+  GtkTextTagPrivate *priv = text_tag->priv;
   GObject *object = G_OBJECT (text_tag);
   PangoFontDescription *new_font_desc;
   PangoFontMask old_mask, new_mask, changed_mask, set_changed_mask;
@@ -897,8 +914,8 @@ set_font_description (GtkTextTag           *text_tag,
   else
     new_font_desc = pango_font_description_new ();
 
-  if (text_tag->values->font)
-    old_mask = pango_font_description_get_set_fields (text_tag->values->font);
+  if (priv->values->font)
+    old_mask = pango_font_description_get_set_fields (priv->values->font);
   else
     old_mask = 0;
   
@@ -907,10 +924,10 @@ set_font_description (GtkTextTag           *text_tag,
   changed_mask = old_mask | new_mask;
   set_changed_mask = old_mask ^ new_mask;
 
-  if (text_tag->values->font)
-    pango_font_description_free (text_tag->values->font);
-  text_tag->values->font = new_font_desc;
-  
+  if (priv->values->font)
+    pango_font_description_free (priv->values->font);
+  priv->values->font = new_font_desc;
+
   g_object_freeze_notify (object);
 
   g_object_notify (object, "font-desc");
@@ -940,8 +957,10 @@ set_font_description (GtkTextTag           *text_tag,
 static void
 gtk_text_tag_ensure_font (GtkTextTag *text_tag)
 {
-  if (!text_tag->values->font)
-    text_tag->values->font = pango_font_description_new ();
+  GtkTextTagPrivate *priv = text_tag->priv;
+
+  if (!priv->values->font)
+    priv->values->font = pango_font_description_new ();
 }
 
 static void
@@ -950,16 +969,15 @@ gtk_text_tag_set_property (GObject      *object,
                            const GValue *value,
                            GParamSpec   *pspec)
 {
-  GtkTextTag *text_tag;
+  GtkTextTag *text_tag = GTK_TEXT_TAG (object);
+  GtkTextTagPrivate *priv = text_tag->priv;
   gboolean size_changed = FALSE;
 
-  text_tag = GTK_TEXT_TAG (object);
-
   switch (prop_id)
     {
     case PROP_NAME:
-      g_return_if_fail (text_tag->name == NULL);
-      text_tag->name = g_value_dup_string (value);
+      g_return_if_fail (priv->name == NULL);
+      priv->name = g_value_dup_string (value);
       break;
 
     case PROP_BACKGROUND:
@@ -1049,44 +1067,44 @@ gtk_text_tag_set_property (GObject      *object,
 	PangoFontMask old_set_mask;
 
 	gtk_text_tag_ensure_font (text_tag);
-	old_set_mask = pango_font_description_get_set_fields (text_tag->values->font);
+	old_set_mask = pango_font_description_get_set_fields (priv->values->font);
  
 	switch (prop_id)
 	  {
 	  case PROP_FAMILY:
-	    pango_font_description_set_family (text_tag->values->font,
+	    pango_font_description_set_family (priv->values->font,
 					       g_value_get_string (value));
 	    break;
 	  case PROP_STYLE:
-	    pango_font_description_set_style (text_tag->values->font,
+	    pango_font_description_set_style (priv->values->font,
 					      g_value_get_enum (value));
 	    break;
 	  case PROP_VARIANT:
-	    pango_font_description_set_variant (text_tag->values->font,
+	    pango_font_description_set_variant (priv->values->font,
 						g_value_get_enum (value));
 	    break;
 	  case PROP_WEIGHT:
-	    pango_font_description_set_weight (text_tag->values->font,
+	    pango_font_description_set_weight (priv->values->font,
 					       g_value_get_int (value));
 	    break;
 	  case PROP_STRETCH:
-	    pango_font_description_set_stretch (text_tag->values->font,
+	    pango_font_description_set_stretch (priv->values->font,
 						g_value_get_enum (value));
 	    break;
 	  case PROP_SIZE:
-	    pango_font_description_set_size (text_tag->values->font,
+	    pango_font_description_set_size (priv->values->font,
 					     g_value_get_int (value));
 	    g_object_notify (object, "size-points");
 	    break;
 	  case PROP_SIZE_POINTS:
-	    pango_font_description_set_size (text_tag->values->font,
+	    pango_font_description_set_size (priv->values->font,
 					     g_value_get_double (value) * PANGO_SCALE);
 	    g_object_notify (object, "size");
 	    break;
 	  }
 
 	size_changed = TRUE;
-	notify_set_changed (object, old_set_mask & pango_font_description_get_set_fields (text_tag->values->font));
+	notify_set_changed (object, old_set_mask & pango_font_description_get_set_fields (priv->values->font));
 	g_object_notify (object, "font-desc");
 	g_object_notify (object, "font");
 
@@ -1094,117 +1112,117 @@ gtk_text_tag_set_property (GObject      *object,
       }
       
     case PROP_SCALE:
-      text_tag->values->font_scale = g_value_get_double (value);
-      text_tag->scale_set = TRUE;
+      priv->values->font_scale = g_value_get_double (value);
+      priv->scale_set = TRUE;
       g_object_notify (object, "scale-set");
       size_changed = TRUE;
       break;
       
     case PROP_PIXELS_ABOVE_LINES:
-      text_tag->pixels_above_lines_set = TRUE;
-      text_tag->values->pixels_above_lines = g_value_get_int (value);
+      priv->pixels_above_lines_set = TRUE;
+      priv->values->pixels_above_lines = g_value_get_int (value);
       g_object_notify (object, "pixels-above-lines-set");
       size_changed = TRUE;
       break;
 
     case PROP_PIXELS_BELOW_LINES:
-      text_tag->pixels_below_lines_set = TRUE;
-      text_tag->values->pixels_below_lines = g_value_get_int (value);
+      priv->pixels_below_lines_set = TRUE;
+      priv->values->pixels_below_lines = g_value_get_int (value);
       g_object_notify (object, "pixels-below-lines-set");
       size_changed = TRUE;
       break;
 
     case PROP_PIXELS_INSIDE_WRAP:
-      text_tag->pixels_inside_wrap_set = TRUE;
-      text_tag->values->pixels_inside_wrap = g_value_get_int (value);
+      priv->pixels_inside_wrap_set = TRUE;
+      priv->values->pixels_inside_wrap = g_value_get_int (value);
       g_object_notify (object, "pixels-inside-wrap-set");
       size_changed = TRUE;
       break;
 
     case PROP_EDITABLE:
-      text_tag->editable_set = TRUE;
-      text_tag->values->editable = g_value_get_boolean (value);
+      priv->editable_set = TRUE;
+      priv->values->editable = g_value_get_boolean (value);
       g_object_notify (object, "editable-set");
       break;
 
     case PROP_WRAP_MODE:
-      text_tag->wrap_mode_set = TRUE;
-      text_tag->values->wrap_mode = g_value_get_enum (value);
+      priv->wrap_mode_set = TRUE;
+      priv->values->wrap_mode = g_value_get_enum (value);
       g_object_notify (object, "wrap-mode-set");
       size_changed = TRUE;
       break;
 
     case PROP_JUSTIFICATION:
-      text_tag->justification_set = TRUE;
-      text_tag->values->justification = g_value_get_enum (value);
+      priv->justification_set = TRUE;
+      priv->values->justification = g_value_get_enum (value);
       g_object_notify (object, "justification-set");
       size_changed = TRUE;
       break;
 
     case PROP_DIRECTION:
-      text_tag->values->direction = g_value_get_enum (value);
+      priv->values->direction = g_value_get_enum (value);
       break;
 
     case PROP_LEFT_MARGIN:
-      text_tag->left_margin_set = TRUE;
-      text_tag->values->left_margin = g_value_get_int (value);
+      priv->left_margin_set = TRUE;
+      priv->values->left_margin = g_value_get_int (value);
       g_object_notify (object, "left-margin-set");
       size_changed = TRUE;
       break;
 
     case PROP_INDENT:
-      text_tag->indent_set = TRUE;
-      text_tag->values->indent = g_value_get_int (value);
+      priv->indent_set = TRUE;
+      priv->values->indent = g_value_get_int (value);
       g_object_notify (object, "indent-set");
       size_changed = TRUE;
       break;
 
     case PROP_STRIKETHROUGH:
-      text_tag->strikethrough_set = TRUE;
-      text_tag->values->appearance.strikethrough = g_value_get_boolean (value);
+      priv->strikethrough_set = TRUE;
+      priv->values->appearance.strikethrough = g_value_get_boolean (value);
       g_object_notify (object, "strikethrough-set");
       break;
 
     case PROP_RIGHT_MARGIN:
-      text_tag->right_margin_set = TRUE;
-      text_tag->values->right_margin = g_value_get_int (value);
+      priv->right_margin_set = TRUE;
+      priv->values->right_margin = g_value_get_int (value);
       g_object_notify (object, "right-margin-set");
       size_changed = TRUE;
       break;
 
     case PROP_UNDERLINE:
-      text_tag->underline_set = TRUE;
-      text_tag->values->appearance.underline = g_value_get_enum (value);
+      priv->underline_set = TRUE;
+      priv->values->appearance.underline = g_value_get_enum (value);
       g_object_notify (object, "underline-set");
       break;
 
     case PROP_RISE:
-      text_tag->rise_set = TRUE;
-      text_tag->values->appearance.rise = g_value_get_int (value);
+      priv->rise_set = TRUE;
+      priv->values->appearance.rise = g_value_get_int (value);
       g_object_notify (object, "rise-set");
       size_changed = TRUE;      
       break;
 
     case PROP_BACKGROUND_FULL_HEIGHT:
-      text_tag->bg_full_height_set = TRUE;
-      text_tag->values->bg_full_height = g_value_get_boolean (value);
+      priv->bg_full_height_set = TRUE;
+      priv->values->bg_full_height = g_value_get_boolean (value);
       g_object_notify (object, "background-full-height-set");
       break;
 
     case PROP_LANGUAGE:
-      text_tag->language_set = TRUE;
-      text_tag->values->language = pango_language_from_string (g_value_get_string (value));
+      priv->language_set = TRUE;
+      priv->values->language = pango_language_from_string (g_value_get_string (value));
       g_object_notify (object, "language-set");
       break;
 
     case PROP_TABS:
-      text_tag->tabs_set = TRUE;
+      priv->tabs_set = TRUE;
 
-      if (text_tag->values->tabs)
-        pango_tab_array_free (text_tag->values->tabs);
+      if (priv->values->tabs)
+        pango_tab_array_free (priv->values->tabs);
 
       /* FIXME I'm not sure if this is a memleak or not */
-      text_tag->values->tabs =
+      priv->values->tabs =
         pango_tab_array_copy (g_value_get_boxed (value));
 
       g_object_notify (object, "tabs-set");
@@ -1213,8 +1231,8 @@ gtk_text_tag_set_property (GObject      *object,
       break;
 
     case PROP_INVISIBLE:
-      text_tag->invisible_set = TRUE;
-      text_tag->values->invisible = g_value_get_boolean (value);
+      priv->invisible_set = TRUE;
+      priv->values->invisible = g_value_get_boolean (value);
       g_object_notify (object, "invisible-set");
       size_changed = TRUE;
       break;
@@ -1243,7 +1261,7 @@ gtk_text_tag_set_property (GObject      *object,
       break;
 
     case PROP_ACCUMULATIVE_MARGIN:
-      text_tag->accumulative_margin = g_value_get_boolean (value);
+      priv->accumulative_margin = g_value_get_boolean (value);
       g_object_notify (object, "accumulative-margin");
       size_changed = TRUE;
       break;
@@ -1251,11 +1269,11 @@ gtk_text_tag_set_property (GObject      *object,
       /* Whether the value should be used... */
 
     case PROP_BACKGROUND_SET:
-      text_tag->bg_color_set = g_value_get_boolean (value);
+      priv->bg_color_set = g_value_get_boolean (value);
       break;
 
     case PROP_FOREGROUND_SET:
-      text_tag->fg_color_set = g_value_get_boolean (value);
+      priv->fg_color_set = g_value_get_boolean (value);
       break;
 
     case PROP_FAMILY_SET:
@@ -1266,8 +1284,8 @@ gtk_text_tag_set_property (GObject      *object,
     case PROP_SIZE_SET:
       if (!g_value_get_boolean (value))
 	{
-	  if (text_tag->values->font)
-	    pango_font_description_unset_fields (text_tag->values->font,
+	  if (priv->values->font)
+	    pango_font_description_unset_fields (priv->values->font,
 						 get_property_font_set_mask (prop_id));
 	}
       else
@@ -1275,95 +1293,95 @@ gtk_text_tag_set_property (GObject      *object,
 	  PangoFontMask changed_mask;
 	  
 	  gtk_text_tag_ensure_font (text_tag);
-	  changed_mask = set_font_desc_fields (text_tag->values->font,
+	  changed_mask = set_font_desc_fields (priv->values->font,
 					       get_property_font_set_mask (prop_id));
 	  notify_fields_changed (G_OBJECT (text_tag), changed_mask);
 	}
       break;
 
     case PROP_SCALE_SET:
-      text_tag->scale_set = g_value_get_boolean (value);
+      priv->scale_set = g_value_get_boolean (value);
       size_changed = TRUE;
       break;
       
     case PROP_PIXELS_ABOVE_LINES_SET:
-      text_tag->pixels_above_lines_set = g_value_get_boolean (value);
+      priv->pixels_above_lines_set = g_value_get_boolean (value);
       size_changed = TRUE;
       break;
 
     case PROP_PIXELS_BELOW_LINES_SET:
-      text_tag->pixels_below_lines_set = g_value_get_boolean (value);
+      priv->pixels_below_lines_set = g_value_get_boolean (value);
       size_changed = TRUE;
       break;
 
     case PROP_PIXELS_INSIDE_WRAP_SET:
-      text_tag->pixels_inside_wrap_set = g_value_get_boolean (value);
+      priv->pixels_inside_wrap_set = g_value_get_boolean (value);
       size_changed = TRUE;
       break;
 
     case PROP_EDITABLE_SET:
-      text_tag->editable_set = g_value_get_boolean (value);
+      priv->editable_set = g_value_get_boolean (value);
       break;
 
     case PROP_WRAP_MODE_SET:
-      text_tag->wrap_mode_set = g_value_get_boolean (value);
+      priv->wrap_mode_set = g_value_get_boolean (value);
       size_changed = TRUE;
       break;
 
     case PROP_JUSTIFICATION_SET:
-      text_tag->justification_set = g_value_get_boolean (value);
+      priv->justification_set = g_value_get_boolean (value);
       size_changed = TRUE;
       break;
       
     case PROP_LEFT_MARGIN_SET:
-      text_tag->left_margin_set = g_value_get_boolean (value);
+      priv->left_margin_set = g_value_get_boolean (value);
       size_changed = TRUE;
       break;
 
     case PROP_INDENT_SET:
-      text_tag->indent_set = g_value_get_boolean (value);
+      priv->indent_set = g_value_get_boolean (value);
       size_changed = TRUE;
       break;
 
     case PROP_STRIKETHROUGH_SET:
-      text_tag->strikethrough_set = g_value_get_boolean (value);
+      priv->strikethrough_set = g_value_get_boolean (value);
       break;
 
     case PROP_RIGHT_MARGIN_SET:
-      text_tag->right_margin_set = g_value_get_boolean (value);
+      priv->right_margin_set = g_value_get_boolean (value);
       size_changed = TRUE;
       break;
 
     case PROP_UNDERLINE_SET:
-      text_tag->underline_set = g_value_get_boolean (value);
+      priv->underline_set = g_value_get_boolean (value);
       break;
 
     case PROP_RISE_SET:
-      text_tag->rise_set = g_value_get_boolean (value);
+      priv->rise_set = g_value_get_boolean (value);
       size_changed = TRUE;
       break;
 
     case PROP_BACKGROUND_FULL_HEIGHT_SET:
-      text_tag->bg_full_height_set = g_value_get_boolean (value);
+      priv->bg_full_height_set = g_value_get_boolean (value);
       break;
 
     case PROP_LANGUAGE_SET:
-      text_tag->language_set = g_value_get_boolean (value);
+      priv->language_set = g_value_get_boolean (value);
       size_changed = TRUE;
       break;
 
     case PROP_TABS_SET:
-      text_tag->tabs_set = g_value_get_boolean (value);
+      priv->tabs_set = g_value_get_boolean (value);
       size_changed = TRUE;
       break;
 
     case PROP_INVISIBLE_SET:
-      text_tag->invisible_set = g_value_get_boolean (value);
+      priv->invisible_set = g_value_get_boolean (value);
       size_changed = TRUE;
       break;
       
     case PROP_PARAGRAPH_BACKGROUND_SET:
-      text_tag->pg_bg_color_set = g_value_get_boolean (value);
+      priv->pg_bg_color_set = g_value_get_boolean (value);
       break;
 
     default:
@@ -1381,8 +1399,8 @@ gtk_text_tag_set_property (GObject      *object,
    * signal here, but the two objects are already tightly bound.
    */
 
-  if (text_tag->table)
-    g_signal_emit_by_name (text_tag->table,
+  if (priv->table)
+    g_signal_emit_by_name (priv->table,
                            "tag_changed",
                            text_tag, size_changed);
 }
@@ -1393,22 +1411,21 @@ gtk_text_tag_get_property (GObject      *object,
                            GValue       *value,
                            GParamSpec   *pspec)
 {
-  GtkTextTag *tag;
-
-  tag = GTK_TEXT_TAG (object);
+  GtkTextTag *tag = GTK_TEXT_TAG (object);
+  GtkTextTagPrivate *priv = tag->priv;
 
   switch (prop_id)
     {
     case PROP_NAME:
-      g_value_set_string (value, tag->name);
+      g_value_set_string (value, priv->name);
       break;
 
     case PROP_BACKGROUND_GDK:
-      g_value_set_boxed (value, &tag->values->appearance.bg_color);
+      g_value_set_boxed (value, &priv->values->appearance.bg_color);
       break;
 
     case PROP_FOREGROUND_GDK:
-      g_value_set_boxed (value, &tag->values->appearance.fg_color);
+      g_value_set_boxed (value, &priv->values->appearance.fg_color);
       break;
 
     case PROP_FONT:
@@ -1416,15 +1433,15 @@ gtk_text_tag_get_property (GObject      *object,
           gchar *str;
 
 	  gtk_text_tag_ensure_font (tag);
-	  
-	  str = pango_font_description_to_string (tag->values->font);
+
+	  str = pango_font_description_to_string (priv->values->font);
           g_value_take_string (value, str);
         }
       break;
 
     case PROP_FONT_DESC:
       gtk_text_tag_ensure_font (tag);
-      g_value_set_boxed (value, tag->values->font);
+      g_value_set_boxed (value, priv->values->font);
       break;
 
     case PROP_FAMILY:
@@ -1438,122 +1455,122 @@ gtk_text_tag_get_property (GObject      *object,
       switch (prop_id)
 	{
 	case PROP_FAMILY:
-	  g_value_set_string (value, pango_font_description_get_family (tag->values->font));
+	  g_value_set_string (value, pango_font_description_get_family (priv->values->font));
 	  break;
 	  
 	case PROP_STYLE:
-	  g_value_set_enum (value, pango_font_description_get_style (tag->values->font));
+	  g_value_set_enum (value, pango_font_description_get_style (priv->values->font));
 	  break;
 	  
 	case PROP_VARIANT:
-	  g_value_set_enum (value, pango_font_description_get_variant (tag->values->font));
+	  g_value_set_enum (value, pango_font_description_get_variant (priv->values->font));
 	  break;
 	  
 	case PROP_WEIGHT:
-	  g_value_set_int (value, pango_font_description_get_weight (tag->values->font));
+	  g_value_set_int (value, pango_font_description_get_weight (priv->values->font));
 	  break;
 	  
 	case PROP_STRETCH:
-	  g_value_set_enum (value, pango_font_description_get_stretch (tag->values->font));
+	  g_value_set_enum (value, pango_font_description_get_stretch (priv->values->font));
 	  break;
 	  
 	case PROP_SIZE:
-	  g_value_set_int (value, pango_font_description_get_size (tag->values->font));
+	  g_value_set_int (value, pango_font_description_get_size (priv->values->font));
 	  break;
 	  
 	case PROP_SIZE_POINTS:
-	  g_value_set_double (value, ((double)pango_font_description_get_size (tag->values->font)) / (double)PANGO_SCALE);
+	  g_value_set_double (value, ((double)pango_font_description_get_size (priv->values->font)) / (double)PANGO_SCALE);
 	  break;
 	}
       break;
       
     case PROP_SCALE:
-      g_value_set_double (value, tag->values->font_scale);
+      g_value_set_double (value, priv->values->font_scale);
       break;
       
     case PROP_PIXELS_ABOVE_LINES:
-      g_value_set_int (value,  tag->values->pixels_above_lines);
+      g_value_set_int (value,  priv->values->pixels_above_lines);
       break;
 
     case PROP_PIXELS_BELOW_LINES:
-      g_value_set_int (value,  tag->values->pixels_below_lines);
+      g_value_set_int (value,  priv->values->pixels_below_lines);
       break;
 
     case PROP_PIXELS_INSIDE_WRAP:
-      g_value_set_int (value,  tag->values->pixels_inside_wrap);
+      g_value_set_int (value,  priv->values->pixels_inside_wrap);
       break;
 
     case PROP_EDITABLE:
-      g_value_set_boolean (value, tag->values->editable);
+      g_value_set_boolean (value, priv->values->editable);
       break;
 
     case PROP_WRAP_MODE:
-      g_value_set_enum (value, tag->values->wrap_mode);
+      g_value_set_enum (value, priv->values->wrap_mode);
       break;
 
     case PROP_JUSTIFICATION:
-      g_value_set_enum (value, tag->values->justification);
+      g_value_set_enum (value, priv->values->justification);
       break;
 
     case PROP_DIRECTION:
-      g_value_set_enum (value, tag->values->direction);
+      g_value_set_enum (value, priv->values->direction);
       break;
       
     case PROP_LEFT_MARGIN:
-      g_value_set_int (value,  tag->values->left_margin);
+      g_value_set_int (value,  priv->values->left_margin);
       break;
 
     case PROP_INDENT:
-      g_value_set_int (value,  tag->values->indent);
+      g_value_set_int (value,  priv->values->indent);
       break;
 
     case PROP_STRIKETHROUGH:
-      g_value_set_boolean (value, tag->values->appearance.strikethrough);
+      g_value_set_boolean (value, priv->values->appearance.strikethrough);
       break;
 
     case PROP_RIGHT_MARGIN:
-      g_value_set_int (value, tag->values->right_margin);
+      g_value_set_int (value, priv->values->right_margin);
       break;
 
     case PROP_UNDERLINE:
-      g_value_set_enum (value, tag->values->appearance.underline);
+      g_value_set_enum (value, priv->values->appearance.underline);
       break;
 
     case PROP_RISE:
-      g_value_set_int (value, tag->values->appearance.rise);
+      g_value_set_int (value, priv->values->appearance.rise);
       break;
 
     case PROP_BACKGROUND_FULL_HEIGHT:
-      g_value_set_boolean (value, tag->values->bg_full_height);
+      g_value_set_boolean (value, priv->values->bg_full_height);
       break;
 
     case PROP_LANGUAGE:
-      g_value_set_string (value, pango_language_to_string (tag->values->language));
+      g_value_set_string (value, pango_language_to_string (priv->values->language));
       break;
 
     case PROP_TABS:
-      if (tag->values->tabs)
-        g_value_set_boxed (value, tag->values->tabs);
+      if (priv->values->tabs)
+        g_value_set_boxed (value, priv->values->tabs);
       break;
 
     case PROP_INVISIBLE:
-      g_value_set_boolean (value, tag->values->invisible);
+      g_value_set_boolean (value, priv->values->invisible);
       break;
       
     case PROP_PARAGRAPH_BACKGROUND_GDK:
-      g_value_set_boxed (value, tag->values->pg_bg_color);
+      g_value_set_boxed (value, priv->values->pg_bg_color);
       break;
 
     case PROP_ACCUMULATIVE_MARGIN:
-      g_value_set_boolean (value, tag->accumulative_margin);
+      g_value_set_boolean (value, priv->accumulative_margin);
       break;
 
     case PROP_BACKGROUND_SET:
-      g_value_set_boolean (value, tag->bg_color_set);
+      g_value_set_boolean (value, priv->bg_color_set);
       break;
 
     case PROP_FOREGROUND_SET:
-      g_value_set_boolean (value, tag->fg_color_set);
+      g_value_set_boolean (value, priv->fg_color_set);
       break;
 
     case PROP_FAMILY_SET:
@@ -1563,7 +1580,7 @@ gtk_text_tag_get_property (GObject      *object,
     case PROP_STRETCH_SET:
     case PROP_SIZE_SET:
       {
-	PangoFontMask set_mask = tag->values->font ? pango_font_description_get_set_fields (tag->values->font) : 0;
+	PangoFontMask set_mask = priv->values->font ? pango_font_description_get_set_fields (priv->values->font) : 0;
 	PangoFontMask test_mask = get_property_font_set_mask (prop_id);
 	g_value_set_boolean (value, (set_mask & test_mask) != 0);
 
@@ -1571,75 +1588,75 @@ gtk_text_tag_get_property (GObject      *object,
       }
 
     case PROP_SCALE_SET:
-      g_value_set_boolean (value, tag->scale_set);
+      g_value_set_boolean (value, priv->scale_set);
       break;
       
     case PROP_PIXELS_ABOVE_LINES_SET:
-      g_value_set_boolean (value, tag->pixels_above_lines_set);
+      g_value_set_boolean (value, priv->pixels_above_lines_set);
       break;
 
     case PROP_PIXELS_BELOW_LINES_SET:
-      g_value_set_boolean (value, tag->pixels_below_lines_set);
+      g_value_set_boolean (value, priv->pixels_below_lines_set);
       break;
 
     case PROP_PIXELS_INSIDE_WRAP_SET:
-      g_value_set_boolean (value, tag->pixels_inside_wrap_set);
+      g_value_set_boolean (value, priv->pixels_inside_wrap_set);
       break;
 
     case PROP_EDITABLE_SET:
-      g_value_set_boolean (value, tag->editable_set);
+      g_value_set_boolean (value, priv->editable_set);
       break;
 
     case PROP_WRAP_MODE_SET:
-      g_value_set_boolean (value, tag->wrap_mode_set);
+      g_value_set_boolean (value, priv->wrap_mode_set);
       break;
 
     case PROP_JUSTIFICATION_SET:
-      g_value_set_boolean (value, tag->justification_set);
+      g_value_set_boolean (value, priv->justification_set);
       break;
       
     case PROP_LEFT_MARGIN_SET:
-      g_value_set_boolean (value, tag->left_margin_set);
+      g_value_set_boolean (value, priv->left_margin_set);
       break;
 
     case PROP_INDENT_SET:
-      g_value_set_boolean (value, tag->indent_set);
+      g_value_set_boolean (value, priv->indent_set);
       break;
 
     case PROP_STRIKETHROUGH_SET:
-      g_value_set_boolean (value, tag->strikethrough_set);
+      g_value_set_boolean (value, priv->strikethrough_set);
       break;
 
     case PROP_RIGHT_MARGIN_SET:
-      g_value_set_boolean (value, tag->right_margin_set);
+      g_value_set_boolean (value, priv->right_margin_set);
       break;
 
     case PROP_UNDERLINE_SET:
-      g_value_set_boolean (value, tag->underline_set);
+      g_value_set_boolean (value, priv->underline_set);
       break;
 
     case PROP_RISE_SET:
-      g_value_set_boolean (value, tag->rise_set);
+      g_value_set_boolean (value, priv->rise_set);
       break;
 
     case PROP_BACKGROUND_FULL_HEIGHT_SET:
-      g_value_set_boolean (value, tag->bg_full_height_set);
+      g_value_set_boolean (value, priv->bg_full_height_set);
       break;
 
     case PROP_LANGUAGE_SET:
-      g_value_set_boolean (value, tag->language_set);
+      g_value_set_boolean (value, priv->language_set);
       break;
 
     case PROP_TABS_SET:
-      g_value_set_boolean (value, tag->tabs_set);
+      g_value_set_boolean (value, priv->tabs_set);
       break;
 
     case PROP_INVISIBLE_SET:
-      g_value_set_boolean (value, tag->invisible_set);
+      g_value_set_boolean (value, priv->invisible_set);
       break;
       
     case PROP_PARAGRAPH_BACKGROUND_SET:
-      g_value_set_boolean (value, tag->pg_bg_color_set);
+      g_value_set_boolean (value, priv->pg_bg_color_set);
       break;
 
     case PROP_BACKGROUND:
@@ -1665,10 +1682,11 @@ typedef struct {
 static void
 delta_priority_foreach (GtkTextTag *tag, gpointer user_data)
 {
+  GtkTextTagPrivate *priv = tag->priv;
   DeltaData *dd = user_data;
 
-  if (tag->priority >= dd->low && tag->priority <= dd->high)
-    tag->priority += dd->delta;
+  if (priv->priority >= dd->low && priv->priority <= dd->high)
+    priv->priority += dd->delta;
 }
 
 /**
@@ -1684,7 +1702,7 @@ gtk_text_tag_get_priority (GtkTextTag *tag)
 {
   g_return_val_if_fail (GTK_IS_TEXT_TAG (tag), 0);
 
-  return tag->priority;
+  return tag->priv->priority;
 }
 
 /**
@@ -1708,34 +1726,38 @@ void
 gtk_text_tag_set_priority (GtkTextTag *tag,
                            gint        priority)
 {
+  GtkTextTagPrivate *priv;
   DeltaData dd;
 
   g_return_if_fail (GTK_IS_TEXT_TAG (tag));
-  g_return_if_fail (tag->table != NULL);
-  g_return_if_fail (priority >= 0);
-  g_return_if_fail (priority < gtk_text_tag_table_get_size (tag->table));
 
-  if (priority == tag->priority)
+  priv = tag->priv;
+
+  g_return_if_fail (priv->table != NULL);
+  g_return_if_fail (priority >= 0);
+  g_return_if_fail (priority < gtk_text_tag_table_get_size (priv->table));
+
+  if (priority == priv->priority)
     return;
 
-  if (priority < tag->priority)
+  if (priority < priv->priority)
     {
       dd.low = priority;
-      dd.high = tag->priority - 1;
+      dd.high = priv->priority - 1;
       dd.delta = 1;
     }
   else
     {
-      dd.low = tag->priority + 1;
+      dd.low = priv->priority + 1;
       dd.high = priority;
       dd.delta = -1;
     }
 
-  gtk_text_tag_table_foreach (tag->table,
+  gtk_text_tag_table_foreach (priv->table,
                               delta_priority_foreach,
                               &dd);
 
-  tag->priority = priority;
+  priv->priority = priority;
 }
 
 /**
@@ -1779,7 +1801,7 @@ tag_sort_func (gconstpointer first, gconstpointer second)
 
   tag1 = * (GtkTextTag **) first;
   tag2 = * (GtkTextTag **) second;
-  return tag1->priority - tag2->priority;
+  return tag1->priv->priority - tag2->priv->priority;
 }
 
 void
@@ -1801,10 +1823,10 @@ _gtk_text_tag_array_sort (GtkTextTag** tag_array_p,
 
     for (i = len-1; i > 0; i--, iter++) {
       maxPtrPtr = tag = iter;
-      prio = tag[0]->priority;
+      prio = tag[0]->priv->priority;
       for (j = i, tag++; j > 0; j--, tag++) {
-        if (tag[0]->priority < prio) {
-          prio = tag[0]->priority;
+        if (tag[0]->priv->priority < prio) {
+          prio = tag[0]->priv->priority;
           maxPtrPtr = tag;
         }
       }
@@ -1989,22 +2011,22 @@ _gtk_text_attributes_fill_from_tags (GtkTextAttributes *dest,
   while (n < n_tags)
     {
       GtkTextTag *tag = tags[n];
-      GtkTextAttributes *vals = tag->values;
+      GtkTextAttributes *vals = tag->priv->values;
 
-      g_assert (tag->table != NULL);
+      g_assert (tag->priv->table != NULL);
       if (n > 0)
-        g_assert (tags[n]->priority > tags[n-1]->priority);
+        g_assert (tags[n]->priv->priority > tags[n-1]->priv->priority);
 
-      if (tag->bg_color_set)
+      if (tag->priv->bg_color_set)
         {
           dest->appearance.bg_color = vals->appearance.bg_color;
 
           dest->appearance.draw_bg = TRUE;
         }
-      if (tag->fg_color_set)
+      if (tag->priv->fg_color_set)
         dest->appearance.fg_color = vals->appearance.fg_color;
-      
-      if (tag->pg_bg_color_set)
+
+      if (tag->priv->pg_bg_color_set)
         {
           dest->pg_bg_color = gdk_color_copy (vals->pg_bg_color);
         }
@@ -2018,72 +2040,72 @@ _gtk_text_attributes_fill_from_tags (GtkTextAttributes *dest,
 	}
 
       /* multiply all the scales together to get a composite */
-      if (tag->scale_set)
+      if (tag->priv->scale_set)
         dest->font_scale *= vals->font_scale;
-      
-      if (tag->justification_set)
+
+      if (tag->priv->justification_set)
         dest->justification = vals->justification;
 
       if (vals->direction != GTK_TEXT_DIR_NONE)
         dest->direction = vals->direction;
 
-      if (tag->left_margin_set) 
+      if (tag->priv->left_margin_set)
         {
-          if (tag->accumulative_margin)
+          if (tag->priv->accumulative_margin)
             left_margin_accumulative += vals->left_margin;
           else
             dest->left_margin = vals->left_margin;
         }
 
-      if (tag->indent_set)
+      if (tag->priv->indent_set)
         dest->indent = vals->indent;
 
-      if (tag->rise_set)
+      if (tag->priv->rise_set)
         dest->appearance.rise = vals->appearance.rise;
 
-      if (tag->right_margin_set) 
+      if (tag->priv->right_margin_set)
         {
-          if (tag->accumulative_margin)
+          if (tag->priv->accumulative_margin)
             right_margin_accumulative += vals->right_margin;
           else
             dest->right_margin = vals->right_margin;
         }
 
-      if (tag->pixels_above_lines_set)
+      if (tag->priv->pixels_above_lines_set)
         dest->pixels_above_lines = vals->pixels_above_lines;
 
-      if (tag->pixels_below_lines_set)
+      if (tag->priv->pixels_below_lines_set)
         dest->pixels_below_lines = vals->pixels_below_lines;
 
-      if (tag->pixels_inside_wrap_set)
+      if (tag->priv->pixels_inside_wrap_set)
         dest->pixels_inside_wrap = vals->pixels_inside_wrap;
 
-      if (tag->tabs_set)
+      if (tag->priv->tabs_set)
         {
           if (dest->tabs)
             pango_tab_array_free (dest->tabs);
           dest->tabs = pango_tab_array_copy (vals->tabs);
         }
 
-      if (tag->wrap_mode_set)
+      if (tag->priv->wrap_mode_set)
         dest->wrap_mode = vals->wrap_mode;
 
-      if (tag->underline_set)
+      if (tag->priv->underline_set)
         dest->appearance.underline = vals->appearance.underline;
 
-      if (tag->strikethrough_set)
+      if (tag->priv->strikethrough_set)
         dest->appearance.strikethrough = vals->appearance.strikethrough;
 
-      if (tag->invisible_set)
+      if (tag->priv->invisible_set)
         dest->invisible = vals->invisible;
 
-      if (tag->editable_set)
+      if (tag->priv->editable_set)
         dest->editable = vals->editable;
 
-      if (tag->bg_full_height_set)
+      if (tag->priv->bg_full_height_set)
         dest->bg_full_height = vals->bg_full_height;
 
-      if (tag->language_set)
+      if (tag->priv->language_set)
 	dest->language = vals->language;
 
       ++n;
@@ -2096,34 +2118,34 @@ _gtk_text_attributes_fill_from_tags (GtkTextAttributes *dest,
 gboolean
 _gtk_text_tag_affects_size (GtkTextTag *tag)
 {
-  g_return_val_if_fail (GTK_IS_TEXT_TAG (tag), FALSE);
+  GtkTextTagPrivate *priv = tag->priv;
 
   return
-    (tag->values->font && pango_font_description_get_set_fields (tag->values->font) != 0) ||
-    tag->scale_set ||
-    tag->justification_set ||
-    tag->left_margin_set ||
-    tag->indent_set ||
-    tag->rise_set ||
-    tag->right_margin_set ||
-    tag->pixels_above_lines_set ||
-    tag->pixels_below_lines_set ||
-    tag->pixels_inside_wrap_set ||
-    tag->tabs_set ||
-    tag->underline_set ||
-    tag->wrap_mode_set ||
-    tag->invisible_set;
+    (priv->values->font && pango_font_description_get_set_fields (priv->values->font) != 0) ||
+    priv->scale_set ||
+    priv->justification_set ||
+    priv->left_margin_set ||
+    priv->indent_set ||
+    priv->rise_set ||
+    priv->right_margin_set ||
+    priv->pixels_above_lines_set ||
+    priv->pixels_below_lines_set ||
+    priv->pixels_inside_wrap_set ||
+    priv->tabs_set ||
+    priv->underline_set ||
+    priv->wrap_mode_set ||
+    priv->invisible_set;
 }
 
 gboolean
 _gtk_text_tag_affects_nonsize_appearance (GtkTextTag *tag)
 {
-  g_return_val_if_fail (GTK_IS_TEXT_TAG (tag), FALSE);
+  GtkTextTagPrivate *priv = tag->priv;
 
   return
-    tag->bg_color_set ||
-    tag->fg_color_set ||
-    tag->strikethrough_set ||
-    tag->bg_full_height_set ||
-    tag->pg_bg_color_set;
+    priv->bg_color_set ||
+    priv->fg_color_set ||
+    priv->strikethrough_set ||
+    priv->bg_full_height_set ||
+    priv->pg_bg_color_set;
 }
