@@ -36,6 +36,7 @@
 #include "gtkcellareabox.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
+#include "gtktypebuiltins.h"
 
 
 /**
@@ -987,7 +988,9 @@ gtk_tree_view_column_update_button (GtkTreeViewColumn *tree_column)
     {
       if (priv->visible)
 	{
-	  gtk_widget_show_now (priv->button);
+          if (gdk_window_is_visible (_gtk_tree_view_get_header_window (GTK_TREE_VIEW (priv->tree_view))))
+            gtk_widget_show_now (priv->button);
+
 	  if (priv->window)
 	    {
 	      if (priv->resizable)
@@ -1327,9 +1330,6 @@ _gtk_tree_view_column_realize_button (GtkTreeViewColumn *column)
   g_return_if_fail (_gtk_tree_view_get_header_window (tree_view) != NULL);
   gtk_widget_set_parent_window (priv->button, _gtk_tree_view_get_header_window (tree_view));
 
-  if (priv->visible)
-    gtk_widget_show (priv->button);
-
   attr.window_type = GDK_WINDOW_CHILD;
   attr.wclass = GDK_INPUT_ONLY;
   attr.visual = gtk_widget_get_visual (GTK_WIDGET (tree_view));
@@ -1515,6 +1515,45 @@ _gtk_tree_view_column_get_cell_at_pos (GtkTreeViewColumn *column,
   return match;
 }
 
+gboolean
+_gtk_tree_view_column_is_blank_at_pos (GtkTreeViewColumn *column,
+                                       GdkRectangle      *cell_area,
+                                       GdkRectangle      *background_area,
+                                       gint               x,
+                                       gint               y)
+{
+  GtkCellRenderer *match;
+  GdkRectangle cell_alloc, aligned_area, inner_area;
+  GtkTreeViewColumnPrivate *priv = column->priv;
+
+  match = _gtk_tree_view_column_get_cell_at_pos (column,
+                                                 cell_area,
+                                                 background_area,
+                                                 x, y);
+  if (!match)
+    return FALSE;
+
+  gtk_cell_area_get_cell_allocation (priv->cell_area,
+                                     priv->cell_area_context,
+                                     priv->tree_view,
+                                     match,
+                                     cell_area,
+                                     &cell_alloc);
+
+  gtk_cell_area_inner_cell_area (priv->cell_area, priv->tree_view,
+                                 &cell_alloc, &inner_area);
+  gtk_cell_renderer_get_aligned_area (match, priv->tree_view, 0,
+                                      &inner_area, &aligned_area);
+
+  if (x < aligned_area.x ||
+      x > aligned_area.x + aligned_area.width ||
+      y < aligned_area.y ||
+      y > aligned_area.y + aligned_area.height)
+    return TRUE;
+
+  return FALSE;
+}
+
 /* Public Functions */
 
 
@@ -2972,7 +3011,7 @@ gtk_tree_view_column_cell_get_position (GtkTreeViewColumn *tree_column,
 					gint              *width)
 {
   GtkTreeViewColumnPrivate *priv;
-  GdkRectangle zero_cell_area = { 0, };
+  GdkRectangle cell_area;
   GdkRectangle allocation;
 
   g_return_val_if_fail (GTK_IS_TREE_VIEW_COLUMN (tree_column), FALSE);
@@ -2980,16 +3019,24 @@ gtk_tree_view_column_cell_get_position (GtkTreeViewColumn *tree_column,
 
   priv = tree_column->priv;
 
-  /* FIXME: Could use a boolean return value for invalid cells */
+  gtk_tree_view_get_background_area (GTK_TREE_VIEW (priv->tree_view),
+                                     NULL, tree_column, &cell_area);
+
   gtk_cell_area_get_cell_allocation (priv->cell_area,
                                      priv->cell_area_context,
                                      priv->tree_view,
                                      cell_renderer,
-                                     &zero_cell_area,
+                                     &cell_area,
                                      &allocation);
 
   if (x_offset)
-    *x_offset = allocation.x;
+    {
+      GdkRectangle button_allocation;
+
+      /* Retrieve column offset */
+      gtk_widget_get_allocation (priv->button, &button_allocation);
+      *x_offset = allocation.x - button_allocation.x;
+    }
   if (width)
     *width = allocation.width;
 
